use std::num::ParseFloatError;

// to construct a `SpecifiedValue` that might be `Present`, use the associated
// function `try_from`. this ensures that `spec` is always a valid specification
// of `value` according to the format discussed at the implementation of
// `TryFrom<String>`
pub enum SpecifiedValue {
    Absent,
    Present {
        spec: String,
        value: f64
    }
}

use SpecifiedValue::{Absent, Present};

impl SpecifiedValue {
    // get the specification for this value. the associated function `try_from`
    // is essentially a left inverse of this method:
    //
    //   SpecifiedValue::try_from(x.spec()) == Ok(x)
    //
    pub fn spec(&self) -> String {
        match self {
            Absent => String::new(),
            Present { spec, .. } => spec.clone()
        }
    }
    
    pub fn is_present(&self) -> bool {
        match self {
            Absent => false,
            Present { .. } => true
        }
    }
}

// we can try to turn a specification string into a `SpecifiedValue`. if the
// specification is empty, the `SpecifiedValue` is `Absent`. if the
// specification parses to a floating-point value `x`, the `SpecifiedValue` is
// `Present`, with a `value` of `x`, and the specification is stored in `spec`.
// these are currently the only valid specifications; any other produces an
// error
impl TryFrom<String> for SpecifiedValue {
    type Error = ParseFloatError;
    
    fn try_from(spec: String) -> Result<Self, Self::Error> {
        if spec.is_empty() {
            Ok(Absent)
        } else {
            spec.parse::<f64>().map(
                |value| Present { spec: spec, value: value }
            )
        }
    }
}